package com.iflytek.cloud;


 
/**
 * <h3>文本语义监听器</h3>
 * <p>通过实现此接口，获取当前文本语义理解的状态和结果</p>
 * <p>合成监听器，请参考{@link SynthesizerListener}</p>
 * <p>识别监听器，请参考{@link SpeechRecognizer}</p>
 * <p>语音语义监听器，请参考{@link SpeechUnderstanderListener}</p>
 *
 * @author <script>gShowAuthor();</script>
 * @since version:1073
 * @see TextUnderstander#understandText(String, TextUnderstanderListener)
 * @see SynthesizerListener
 * @see SpeechRecognizer
 * @see SpeechUnderstanderListener
 */
public interface TextUnderstanderListener {
	/**
	 * <h4>返回结果</h4>
	 * <p>
	 * 		返回的结果可能为null，请增加判断处理。
	 * </p>
	 * <p>
	 * 		一次会话中的语义理解结果会一次性返回。当结果返回时，本次会话结束。当出现
	 * 错误，或应用层调用{@link TextUnderstander#cancel}取消当次会话时，在当次会
	 * 话过程可能不会回调此函数。
	 * </p>
	 * 
	 * @param result 结果数据
	 * 
	 * @see #onError
	 * @see TextUnderstander#cancel
	 * @see UnderstanderResult
	 */
	void onResult(UnderstanderResult result);

	/**
	 * <h4>错误回调</h4>
	 * <p>
	 * 		当此函数回调时，说明当次会话出现错误，会话自动结束。
	 * </p>
	 * @param error 错误信息
	 * 
	 * @see SpeechError
	 * @see #onResult
	 * @see TextUnderstander#understandText(String, TextUnderstanderListener)
	 */
	void onError(SpeechError error);
    
}
