package com.iflytek.cloud.msc.ise;

import java.io.UnsupportedEncodingException;
import java.util.Date;

import android.content.Context;
import android.text.TextUtils;

import com.iflytek.cloud.ErrorCode;
import com.iflytek.cloud.SpeechError;
import com.iflytek.cloud.msc.module.MscHandler;
import com.iflytek.cloud.msc.module.MscSession;
import com.iflytek.cloud.msc.util.log.DebugLog;
import com.iflytek.cloud.msc.util.log.PerfLogger;
import com.iflytek.cloud.param.ParamBuilder;
import com.iflytek.msc.MSC;
import com.iflytek.msc.MSCSessionInfo;

public class IseSession extends MscSession{
	private final static int ISE_AUDIO_SAMPLE_CONTINUE = 0x02;
	private final static int ISE_AUDIO_SAMPLE_LAST = 0x04;
	
	/** member properties */
	private MSCSessionInfo	mSessionOut	= new MSCSessionInfo();
	private MSCSessionInfo	mWriteout	= new MSCSessionInfo();
	private byte[] mResult = null;

	/**
	 * Start a ISE session with MSC service
	 */
	@Override
	public int sessionBegin(Context context,
			String userModelId, MscHandler looper) throws SpeechError,
			UnsupportedEncodingException {
		//reset internal state
		mClientID = null;
		
		String iseParams = ParamBuilder.composeIseSessionParam(context, looper);
		
		//TODO : userModleId occupy a location, for future use
		PerfLogger.appendInfo(PerfLogger.MSC_SESSION_BIGNE, null);
		DebugLog.LogD( "QISESessionBegin enter" ); 
		if (TextUtils.isEmpty(userModelId)) {
			mClientID = MSC.QISESessionBegin(iseParams.getBytes(looper.getParamEncoding()), 
					null, mSessionOut);
		} else {
			mClientID = MSC.QISESessionBegin(iseParams.getBytes(looper.getParamEncoding()), 
					userModelId.getBytes(looper.getParamEncoding()), mSessionOut);
			DebugLog.LogD("sessionBegin userModelId:" + userModelId);
		}
		DebugLog.LogD( "QISESessionBegin leave: "+ mSessionOut.errorcode ); 
		PerfLogger.appendInfo(PerfLogger.SESSION_BEGIN_END, null);
		
		//Error handle
		int errorCode = mSessionOut.errorcode;
		//增加防止10109、10132错误直接返回
		if (errorCode != 0 && (errorCode != ErrorCode.MSP_ERROR_CREATE_HANDLE
				&& errorCode != ErrorCode.MSP_ERROR_OVERFLOW && errorCode != ErrorCode.MSP_ERROR_INVALID_OPERATION)) {
			throw new SpeechError(errorCode);
		}
		return errorCode;
	}

	/**
	 * End a ISE session with MSC service
	 * 
	 * @param reason
	 */
	@Override
	public void sessionEnd(String reason) {
		if (null == mClientID) {
			return;
		}
		if (TextUtils.isEmpty(reason)) {
			reason = "unknown";
		}
		DebugLog.LogD("ISESessionEnd enter ");
		long d1 = System.currentTimeMillis();
		int nRet = MSC.QISESessionEnd(mClientID, reason.getBytes());
		DebugLog.LogD("ISESessionEnd leave: " + nRet + " time:"
						+ (System.currentTimeMillis() - d1));
		mClientID = null;
		mSessionID = null;
	}

	/**
	 * Write text-data to MSC, will call the method of MSC.QISETextPut(final char*. final char*)
	 * 
	 * @param data text that will be written to MSC
	 * @throws SpeechError
	 */
	public synchronized void putText(byte[] data, byte[] params) throws SpeechError {
		//write data to MSC
		DebugLog.LogD("QISETextPut enter");
		int error = MSC.QISETextPut(mClientID, data, params);
		DebugLog.LogD("QISETextPut leave: " + error);
		
		//error handle
		if (error != 0) {
			throw new SpeechError(error);
		}
	}
	
	/**
	 * Upload data to MSC, will call the method of MSC.QISEAudioWrite(
	 * final char[], final byte[], int, int, MSCSessionInfo ).
	 * 
	 * @param waveData
	 * @param waveLen
	 * @throws SpeechError
	 */
	public synchronized void pushAudioData(final byte[] waveData, int waveLen) 
			throws SpeechError {
		writeData(waveData, waveLen, ISE_AUDIO_SAMPLE_CONTINUE);
	}
	
	public synchronized void pushEndFlag() throws SpeechError {
		byte[]  data = new byte[0];
    	PerfLogger.appendInfo(PerfLogger.LAST_DATA_FLAG, null);
    	DebugLog.LogD( "IseSession pushEndFlag" );
		writeData(data, 0, ISE_AUDIO_SAMPLE_LAST);
	}

	/**
	 * upload data to server, this method will call MSC.QISEAudioWrite(..)
	 * 
	 * @param waveData
	 * @param waveLen
	 * @param audioStatus
	 * @return
	 * @throws SpeechError
	 */
	private synchronized void writeData(final byte[] waveData, int waveLen, 
			int audioStatus) throws SpeechError {
		DebugLog.LogI("QISEAudioWrite enter, length: "+waveLen );
		int nRet = MSC.QISEAudioWrite(mClientID, waveData, waveLen, audioStatus, mWriteout);
		DebugLog.LogI("QISEAudioWrite leave: " + nRet);
		
		mSessionOut.sesstatus = mWriteout.sesstatus;

		if (0 != nRet) {
			throw new SpeechError(mWriteout.errorcode);
		}
	}

	/**
	 * 
	 * @return true if find record end point
	 */
	synchronized public int getEpStatus() {
		int eps = mWriteout.epstatues;
		return eps;
	}

	/**
	 * Get audio data volume value from MSC, this will call MSC.QISEGetParam to
	 * get the wanted value
	 * 
	 * @return
	 */
	public synchronized int getAudioVolume() {
		if (mClientID == null) {
			return 0;
		}
		
		int ret = 0;
		int vadret = 0;
		try {
			vadret = MSC.QISEGetParam(mClientID, "volume".getBytes(), mWriteout);
			if (vadret == 0) {
				String volume = new String(mWriteout.buffer);
				ret = Integer.parseInt(new String(volume));
			} else {
				DebugLog.LogI("VAD CHECK FALSE");
			}
		} catch (Throwable e) {
			DebugLog.LogI("getAudioVolume Exception vadret = " + vadret);
		}
		return ret;
	}
	
	/**
	 * Get audio data volume value
	 * 
	 * @return
	 */
	synchronized public int getIntValue(String tag) {
		if(mClientID == null) {
			return 0;
		}
			
		int ret = 0;
		try {
			String value = getStringValue(tag);
			if (!TextUtils.isEmpty(value)) {
				ret = Integer.parseInt(new String(value));
			} else {
				//Do nothing
			}
		} catch (Exception e) {
			DebugLog.LogE(e);
		}
		return ret;
	}
	
	/**
	 * ISE module do not support to modify session parameters, no method of
	 * MSC.QISESetParam(..).
	 * 
	 * @return
	 */
//	public synchronized  boolean setParam(String key, String value) {
//		if(TextUtils.isEmpty(key) || TextUtils.isEmpty(value) 
//				|| mSessionID == null) {
//			return false;
//		}
//			
//		DebugLog.LogD("setParam:" + key + "," + value);
//		int ret = -1;
//		try {
//			ret = MSC.QISESetParam(mSessionID, 
//					     key.getBytes(DataUtil.UTF8), 
//					     value.getBytes(DataUtil.UTF8));
//		} catch (UnsupportedEncodingException e) {
//			DebugLog.LogE(e);
//		}
//		return (ret == 0);
//	}
	
	/**
	 * get audio data volume value
	 * 
	 * @return
	 */
	synchronized public String getStringValue(String tag) {
		if(mClientID == null) {
			return null;
		}
			
		int code = 0;
		try {
			code = MSC.QISEGetParam(mClientID, tag.getBytes(), mSessionOut);
			if (code == 0) {
				return new String(mSessionOut.buffer);
			} else {
				//Do nothing
			}
		} catch (Exception e) {
			//Do nothing
		}
		return null;
	}
	
	/**
	 * Get flow-information about upload and download
	 *  
	 * @param tag
	 * @return
	 */
	public static int getIntByTag(String tag)
	{
		try {
			String val = getStringByTag(tag);
			if(TextUtils.isEmpty(val)) {
				return 0;
			}
			return Integer.parseInt(val);
		} catch (Exception e) {
			//Do nothing
		}
		return 0;
	}
	
	/**
	 * Get flow-information about upload and download
	 * 
	 * @param tag
	 * @return
	 */
	public static String getStringByTag(String tag) {
		int code = 0;
		try {
			MSCSessionInfo info = new MSCSessionInfo();
			//TODO : not clear, why QISEGetParam do not need mSessionID?
			code = MSC.QISEGetParam(null, tag.getBytes(), info);
			if (code == 0) {
				return new String(info.buffer).trim();
			} else {
				//Do nothing
			}
		} catch (Exception e) {
			//Do nothing
		}
		return null;
	}

	/**
	 * if engine has result,return true
	 * 
	 * @return
	 */
	synchronized public boolean hasResult() {
		return mSessionOut.sesstatus == 0;
	}

	public byte[] getResultData() {
		return mResult;
	}

	public ResultStatus getResult() throws SpeechError{
		Date d1 = new Date();
		mResult = MSC.QISEGetResult(mClientID, mSessionOut);
		Date d2 = new Date();
		DebugLog.LogI("QISRGetResult leave: " + (null != mResult) + " time:"
						+ (d2.getTime() - d1.getTime()));

		int error = mSessionOut.errorcode;
		if (error == 0) {
			int resultStatus = mSessionOut.rsltstatus;
			switch (resultStatus) {
			case 0:
				DebugLog.LogD("ResultStatus: hasResult" + resultStatus);
				return ResultStatus.hasResult;
			case 2:
				DebugLog.LogI("ResultStatus: noResult" + resultStatus);
				return ResultStatus.noResult;
			case 5:
				DebugLog.LogD("ResultStatus: resultOver" + resultStatus);
				return ResultStatus.resultOver;
				
			default:
				DebugLog.LogD( "IseSession getResult get unmatched result status: "+resultStatus );
			}//end of switch
			
			return ResultStatus.noResult;
		}else{
			DebugLog.LogE( "Result: error " + error );
			throw new SpeechError(error);
		}//end of if-else has no error
	}
	/**
	 * 获得服务端ID
	 */
	protected String getSessionID()
	{
		if(null == mSessionID )
			mSessionID = getStringValue(TAG_SID);
		return mSessionID;
	}
}
