package com.iflytek.cloud.msc.ist;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;

import android.content.Context;
import android.media.MediaRecorder;
import android.media.MediaRecorder.AudioSource;
import android.os.Bundle;
import android.os.HandlerThread;
import android.os.Message;
import android.os.SystemClock;
import android.text.TextUtils;

import com.iflytek.cloud.msc.module.MscHandler;
import com.iflytek.cloud.msc.module.MscSession.ResultStatus;
import com.iflytek.cloud.msc.module.SessPerflog;
import com.iflytek.cloud.msc.util.DataUtil;
import com.iflytek.cloud.msc.util.NetworkUtil;
import com.iflytek.cloud.msc.util.log.DebugLog;
import com.iflytek.cloud.msc.util.log.PerfLogger;
import com.iflytek.cloud.param.HashParam;
import com.iflytek.cloud.param.MscKeys;
import com.iflytek.cloud.record.PcmRecorder;
import com.iflytek.cloud.record.PcmRecorder.PcmRecordListener;
import com.iflytek.cloud.ErrorCode;
import com.iflytek.cloud.TranscripterListener;
import com.iflytek.cloud.TranscripterResult;
import com.iflytek.cloud.SpeechConstant;
import com.iflytek.cloud.SpeechError;
import com.iflytek.cloud.SpeechEvent;

/**
 * Transcript thread,manager record and msc interface
 * 
 * @author iflytek
 * 
 */

public class MscTranscripter extends MscHandler implements PcmRecordListener {
	// 外部回调接口
	protected volatile 	 TranscripterListener mListener = null;

	// 第一次获取到录音数据
	protected boolean       mFirstRecordAudio  = false;
	//首次获取到语音写入MSC时间
	protected boolean    	mFirstDataWrite  = false;
	//获取到第一个识别结果
	protected boolean    	mFirstDataArrival  = false;
	// 是否用户主动点击了停止录音
	protected boolean    	mUstopRecord = false;
	// 录音源
	protected int           mAudioSource = AudioSource.MIC;
	
	protected IstSession  mSession = new IstSession();
	protected PcmRecorder mRecorder = null;
	
	// 会话产生的上下行流量
	public static int     mUpflow =  0;
	public static int     mDownflow = 0;
	
	/*识别结果列表,保存的为原始结果*/
	protected ArrayList<String> mResults = null;
	
	/******************************************************************/
	//会话信息
	private byte[] mBuffer = null;
	private int mLastMscOffset = 0;	//上次写入MSC的音频字节
	private int mLastServerOffset = 0;	//上次服务器返回的已上传字节
	private int mResultSecondLeft = -1;	//结果等待时间
	private String mSessionID = null;	//音频ID
	private int mSyncID = 0;
	
	private String mFilePath = null;
	private AudioAccessor mAccesser = null;
	private final String KEY_TOTAL_AUDIO = "total";
	//会话重试
	private int mMaxSessionTry = MscKeys.VALUE_IST_SESSION_TRY_DEFAULT;	//最大会话尝试次数
	private final int SESSION_RETRY_PERIOD = 500;
	
	//结果等待
	protected final static int MSG_CUSTOM_MIN	= MscHandler.MSG_END+0x00008000;
	private static final int MSG_WAIT_RESULT = MSG_CUSTOM_MIN+1;
	private static final int WAIT_RESULT_PERIOD = 500;

	//会话状态
	private static final int STATUS_NORESULT = 2;
	private static final int STATUS_HASRESULT = 0;
	private static final int STATUS_LASTRESULT = 5;
	
	private boolean mIsProcRecordDataOn = false;
	private boolean mIsRecording = false;
	private SpeechError mError = null;
	private boolean mIsFirstSession = true;
	private boolean mIsRestarting = false;
	
	//网络数据查询延时
	private static final int NET_PERF_MSG_DELAY = 500;
	
	private boolean mUseBluetooth = false;
	/******************************************************************/
	// 性能结构化日志
	protected SessPerflog mPerflog = new SessPerflog();
	
	public MscTranscripter(Context context,HashParam param,HandlerThread mHandlerThread) {
		super( context, mHandlerThread);
		mResults = new ArrayList<String>();
		mUstopRecord = false;
		setParams(param);
	}
	
	/**
	 * 获取录音源
	 * @return 示例：-1、MIC
	 */
	public int getAudioSource()
	{
		return mAudioSource;
	}
	
	/**
	 * 通知音量变化
	 * @param data
	 * @param volume
	 */
	public void callbackVolume(byte[] data, int volume) {
		if (isRunning())
		{
			mListener.onVolumeChanged(volume,data);
		}
	}
	

	/**
	 * 开始识别
	 * @param listener 转写监听器
	 */
	public synchronized void startTranscripting(TranscripterListener listener) {
		this.mListener = listener;
	
		DebugLog.LogD("startTranscripting called");
		// 启动会话
		start();
	}

	@Override
	protected void start() {
		mPerflog.startSession(getParam());
		super.start();
	}
	
	@Override
	protected void onMsgProcess(Message msg) throws Throwable, SpeechError {
		super.onMsgProcess(msg);
		
		if( MSG_START!=msg.what 
				&& !this.isRunning() ){
			return;
		}
		
		switch( msg.what ){
			case MSG_START:
				proc_Msg_Start();
				break;
			case MSG_SESSION_BEGIN:
				proc_Msg_Session_Begin();
				break;
			case MSG_RECORD_AUDIO:
				proc_Msg_Record_Data(msg);
				break;
			case MSG_RECORD_STOPD:
				proc_Msg_Record_Stoped();
				break;
			case MSG_WAIT_RESULT:
				proc_Msg_Wait_Result();
				break;
			case MSG_RESULT:
				proc_Msg_Result(msg);
				break;
			case MSG_NET_PERF:
				proc_Msg_Netperf();
				break;
			case MSG_REQUEST_RESULT:
				requestResult();
				break;
			default:
				break;
			}
	}

	/**
	 * 启动消息处理
	 * @throws Exception
	 */
	protected void proc_Msg_Start() throws Exception {	
		DebugLog.LogD("start connecting");
		String engine = getParam().getString(SpeechConstant.ENGINE_TYPE);
		if(SpeechConstant.TYPE_CLOUD.equals(engine) 
				&& (MscKeys.VALUE_AUDIO_EXTERNAL==this.getAudioSource()||MscKeys.VALUE_AUDIO_EXTERNALFILE==this.getAudioSource()) )
			NetworkUtil.checkNetwork(mContext);
		
		int read_rate = getParam().getInt(MscKeys.KEY_RECORD_READ_RATE, PcmRecorder.READ_INTERVAL40MS);
		
		if( MscKeys.VALUE_AUDIO_EXTERNAL == this.mAudioSource ){
			this.mAccesser = AudioAccessor.createBufferAccessor();
		}else if( MscKeys.VALUE_AUDIO_EXTERNALFILE == this.mAudioSource ){
			this.mAccesser = AudioAccessor.createReadOnlyAccessor(mFilePath);
			//设置音频文件的采样率
			getParam().putParam( SpeechConstant.SAMPLE_RATE, 
					this.mAccesser.getAudioInfo(AudioAccessor.AudioKeys.RATE), 
					false );
		}else{
			this.mAccesser = AudioAccessor.createWriteReadAccessor( mFilePath, this.mSampleRate );
			
			if( null != this.mListener ){
				Bundle bundle = new Bundle();
				bundle.putString(SpeechConstant.IST_AUDIO_PATH, mAccesser.getFilePath() );
				this.mListener.onEvent(SpeechEvent.EVENT_IST_AUDIO_FILE, 0, 0,bundle);
			}
		}
		this.mBuffer = new byte[ mAccesser.getBufferLength() ];
		
		//开启录音
		if(mAudioSource >= AudioSource.DEFAULT && isRunning()){
			DebugLog.LogD( "start  record");
			mUseBluetooth = getParam().getBoolean( SpeechConstant.BLUETOOTH, mUseBluetooth );
			if( mUseBluetooth ){
				startBluetooth();
			}
			
			mRecorder = new PcmRecorder(getSampleRate(), read_rate,mAudioSource);
			mPerflog.pushSessionInfo(SessPerflog.KEY_REC_OPEN);
			mRecorder.startRecording(this);

			if (null != mListener){
				mListener.onBeginOfSpeech();
			}
		}
		
		mPerflog.pushSessionInfo(SessPerflog.KEY_APP_SSB);
		// 插入到消息头部，防止音频被预先处理
		sendMsg(MSG_SESSION_BEGIN,Priority.max,false,0);
		mIsRecording = true;
	}
	
	private int mSessionRetryCount = 0;
	
	/**
	 * MSC SessionBegin消息处理
	 * 对转写来说，直接保存音频，所在会话开始可以在音频缓冲之后
	 * @throws Exception
	 */
	protected void proc_Msg_Session_Begin() throws Exception {
		if( this.isSessionOn() ){
			return;
		}
		
		assertSessionTimes();
		
		int code = 0;
		try{
			PerfLogger.appendInfo(PerfLogger.SDK_SESSION_BIGNE, null);
			code = mSession.sessionBegin(mContext, null, this);
		}catch( SpeechError speechError ){
			speechError.printStackTrace();
			code = speechError.getErrorCode();
			mError = speechError;
		}finally{
			mSessionRetryCount++;
			
			if(code == 0 && null != mSession.mClientID){
				mIsRestarting = false;//重置重启会话标志
				
				if( mIsFirstSession ){
					mIsFirstSession = false;
					//会话开始事件
					if( null != this.mListener ){
						this.mListener.onEvent( SpeechEvent.EVENT_SESSION_BEGIN, 0, 0, null );
					}
				}
				
			}else{
				if( isRunning() ){
					if( this.isRecordMode() ){
//						对转写来说，首先要保存音频，会话开始可延迟
						sendMsg( MSG_SESSION_BEGIN, Priority.normal, false, SESSION_RETRY_PERIOD );
					}else{
						sendMsg(MSG_SESSION_BEGIN, Priority.max,false,0);
					}
					
				}
			}
		}
		
		
	}
	
	/**
	 * 录音数据通知消息
	 * @param msg
	 * @throws Exception
	 */
	protected void proc_Msg_Record_Data(Message msg) throws Exception{
		int dataLen = 0;
		try {
			dataLen = getAudioData();
			
		} catch (IOException e) {
			DebugLog.LogE(e);
			throw new SpeechError( ErrorCode.MSP_ERROR_ACCESS, "Read file Error"+e.getLocalizedMessage() );
		}
		
		try{
			if( isSessionOn() ){
				if( null!=mBuffer && dataLen>0 ){
					uploadData(mBuffer, dataLen, false);
					this.mLastMscOffset += dataLen;
					
					DebugLog.LogD( "LastMscOffset: "+this.mLastMscOffset );
				}
				
				//更新当前会话信息
				this.updateUploadBytes();
				this.updateSessionID();
				this.updateSyncID();
				this.updateCatchLeft();

				sendMsg(obtainMessage(MSG_REQUEST_RESULT,null),Priority.normal, false, 100);
			}
			
		}catch( SpeechError error ){
			mError = error;
			if( ErrorCode.MSP_ERROR_TIME_OUT == error.getErrorCode() ){
				restartSession();
			}else{
				throw( error );
			}
			
		}finally{
			if( this.hasMoreData() ){
				sendMsg(obtainMessage(MSG_RECORD_AUDIO,null));
			}else if( isSessionOn() ){
				stopWriteAutio();
			}
		}//end of try-catch
		
	}
	
	/**
	 * 停止录音
	 * @return
	 */
	public synchronized boolean stopTranscripting(boolean userStop) {
		DebugLog.LogD("stopTranscript, current status is :" + getStatus()+" usercancel : " +userStop);
		mPerflog.pushSessionInfo(SessPerflog.KEY_APP_STOP);
		releaseRecord();
		// 用户主动点击录音停止
		mUstopRecord = userStop;
		sendMsg(MSG_RECORD_STOPD);
		return true;
	}
	
	/**
	 * 录音停止的通知消息
	 * @throws SpeechError
	 * @throws IOException
	 * @throws InterruptedException
	 */
	private void proc_Msg_Record_Stoped() throws SpeechError, IOException, InterruptedException{	
	
		DebugLog.LogD( "recording stop" );
		releaseRecord();

		mPerflog.pushSessionInfo(SessPerflog.KEY_APP_LAU);
		
		// 最低优先级消息，会被自动清除
		
		// 无论是否是用户取消，都回调结束录音接口。因为转写只能由用户取消。
		if ( mListener!=null ){
			mListener.onEndOfSpeech();
		}
	}
	
	/**
	 * 取消识别
	 */
	@Override
	public void cancel(boolean notifyError) {
		if(notifyError && isRunning() && mListener != null){
			mListener.onError(new SpeechError(ErrorCode.ERROR_INTERRUPT));
		}
		releaseRecord();
		// 用户主动点击取消
		if(getStatus() == Status.recording){
			mUstopRecord = true;
		}
		super.cancel(notifyError);
	}
	
	/**
	 * 返回外部当前msc的网络连接状态值，lua版本中可见
	 * 100ms返回一次数据
	 */
	public void proc_Msg_Netperf()
	{
		// NET_PERF_MSG_DELAY ms返回一次
		if(isRunning())
		{
			int netpref = mSession.getIntValue(TAG_NETPERF);
			if(mListener != null)
				mListener.onEvent(SpeechEvent.EVENT_NETPREF, netpref, 0,null);
			// 继续开启下一次通知
			sendMsg(MSG_NET_PERF, Priority.normal,false,NET_PERF_MSG_DELAY);
		}
	}
	
	@Override
	protected void onEnd(SpeechError error) {
		DebugLog.LogD("onSessionEnd");
		releaseRecord();
		mUpflow = mSession.getIntValue(TAG_UPFLOW);
		mDownflow = mSession.getIntValue(TAG_DOWNFLOW);
		getSessionID();

		//更新当前会话信息
		this.updateUploadBytes();
		this.updateSessionID();
		this.updateSyncID();
		
		//会话结束事件
		if( null != this.mListener ){
			Bundle bundle = new Bundle();
			bundle.putString( SpeechConstant.IST_SESSION_ID, this.mSessionID );
			bundle.putInt( SpeechConstant.IST_SYNC_ID, this.mSyncID );
			bundle.putInt( SpeechConstant.IST_AUDIO_UPLOADED, this.mLastServerOffset );
			bundle.putInt( KEY_TOTAL_AUDIO, (int)this.getDataLength() );
			bundle.putString( SpeechConstant.IST_AUDIO_PATH, this.getAudioPath() );
			bundle.putSerializable( "error", error );
			
			this.mListener.onEvent( SpeechEvent.EVENT_SESSION_END, 0, 0, bundle );
		}
		
		try{
			closeAudioFile();
		}catch( Exception e ){
			if( null == error ){
				error = new SpeechError(e);
			}
		}

		// 如果无结果，判断是否需要通知10118错误,默认不返回
		if(mResults.size() <= 0 && error == null 
			&& getParam().getBoolean(SpeechConstant.ASR_NOMATCH_ERROR, true))
		{
			error = new SpeechError(ErrorCode.MSP_ERROR_NO_DATA);
		}
		//结构化日志增加应用层错误码，成功默认返回0
		if(null != error)
			mPerflog.pushSessionInfo( SessPerflog.KEY_APP_RET, error.getErrorCode(), false );
		else
			mPerflog.pushSessionInfo( SessPerflog.KEY_APP_RET, 0, false );
		
//		mLastError = new SpeechError(SpeechError.UNKNOWN,11200);
		
		// 写入录音启动时间、用户是否主动停止等消息
		
		mPerflog.pushSessionInfo(SessPerflog.KEY_REC_USTOP, mUstopRecord ? "1" : "0",false);
		mSession.setParam(SessPerflog.KEY_SESSIONINFO, mPerflog.getSessionInfo());

		PerfLogger.appendInfo(PerfLogger.SESSION_END_BEGIN, null);
		if(mUserCancel)
			mSession.sessionEnd("user abort");
		else if(error != null)
			mSession.sessionEnd("error" + error.getErrorCode());
		else
			mSession.sessionEnd("success");
		PerfLogger.appendInfo(PerfLogger.SESSION_END_END, null);
		
		super.onEnd(error);
		
		if(mListener != null)
		{
			if(mUserCancel)	// cancel by user
			{
				DebugLog.LogD("TranscripterListener#onCancel");
				// mListener.onCancel();
			}else	// auto exit
			{
				DebugLog.LogD("TranscripterListener#onEnd");
				if(error != null)
					mListener.onError(error);
			}
		}
	}

	/**
	 * 通知外部识别结果
	 * @param isLast
	 * @throws SpeechError
	 * @throws UnsupportedEncodingException
	 */
	private void notifyResult(boolean isLast, byte[] resultByte) throws SpeechError, UnsupportedEncodingException{
		String result = null;
		mStatusBegin = SystemClock.elapsedRealtime();
		if(resultByte != null && resultByte.length > 0)
		{
			result = new String(resultByte, DataUtil.UTF8);
		}else if(mResults.size() <= 0)
		{
			//throw new SpeechError(ErrorCode.MSP_ERROR_NO_DATA);
		}else 
		{
			result = "";
		}
		
		//需要放在结果回调前面，确保回调最后结果时已经拿到sessionID
		getSessionID();
		
		// 保存识别结果,用以判断是否返回10118错误
		mResults.add(result);
		if(null != mListener && isRunning())
		{
			// 原始结果
			TranscripterResult reresult = new TranscripterResult(result);
			PerfLogger.appendInfo(PerfLogger.GET_RESULT, null);
			mListener.onResult(reresult,isLast);
		}
		
		DebugLog.LogD("msc result time:" + System.currentTimeMillis());
		

		if(isLast)
		{
			exit(null);
		}
	}
	
	/**
	 * 得到识别结果，不进行解析
	 * @return
	 */
	public TranscripterResult getTranscriptResult() throws SpeechError
	{
		TranscripterResult result = null;
		try {
			if(mSession.getResultData() != null)
			{
				result = new TranscripterResult(new String(mSession.getResultData(),DataUtil.UTF8));
			}
		} catch (UnsupportedEncodingException e) {
			DebugLog.LogE(e);
			throw new SpeechError(ErrorCode.ERROR_INVALID_RESULT);
		}
		return result;
	}
	
	
	@Override
	protected void onParseParam() {
		mAudioSource =  getParam().getInt(SpeechConstant.AUDIO_SOURCE, MediaRecorder.AudioSource.MIC);
		
		// 会话重试次数
		mMaxSessionTry = getParam().getInt( SpeechConstant.IST_SESSION_TRY, mMaxSessionTry );
		
		// 会话信息
		mLastServerOffset = this.getParam().getInt(	SpeechConstant.IST_AUDIO_UPLOADED, 0 );
		mLastMscOffset = mLastServerOffset;
		mSessionID = this.getParam().getString(SpeechConstant.IST_SESSION_ID, null);
		mFilePath = new String( getParam().getString(SpeechConstant.IST_AUDIO_PATH, MscKeys.VALUE_IST_AUDIO_PATH) );
		mSyncID = this.getParam().getInt( SpeechConstant.IST_SYNC_ID, 0 );
		
		super.onParseParam();
	}

	/**
	 * stop listen message
	 * @throws SpeechError 
	 * @throws IOException 
	 * @throws InterruptedException 
	 */
	private void releaseRecord(){
		// stop audio record
		if (mRecorder != null){
			mRecorder.stopRecord(getParam().getBoolean(MscKeys.RECORD_FORCE_STOP, false));
			mRecorder = null;
			mPerflog.pushSessionInfo(SessPerflog.KEY_REC_CLOSE);
			
			try {
				if( null != this.mAccesser ){
					this.mAccesser.flush();
				}
			} catch (Exception e) {
				exit(new SpeechError(e));
				return;
			}
			
			//重置会话重试次数为0
			mSessionRetryCount = 0;
			
			if( mUseBluetooth ){
				stopBluetooth();
			}
		}
		
		mIsRecording = false;
	}
	/**
	 * 结果通知消息
	 * 	status:   2----unknown 0----has result 5----is last result
	 * @throws SpeechError
	 * @throws InterruptedException
	 * @throws UnsupportedEncodingException
	 */
	void proc_Msg_Result(Message msg) throws SpeechError, InterruptedException, UnsupportedEncodingException{
		
		int status = msg.arg1;
		byte[] data = (byte[])msg.obj;

		switch (status) {
			case STATUS_NORESULT:
				break;
			case STATUS_HASRESULT:
				if (!mFirstDataArrival) {
					mFirstDataArrival = true;
					mPerflog.pushSessionInfo(SessPerflog.KEY_APP_FRS);
				}
				notifyResult(false, data);
				break;
			case STATUS_LASTRESULT:
				if (!mFirstDataArrival) {
					mFirstDataArrival = true;
					mPerflog.pushSessionInfo(SessPerflog.KEY_APP_FRS);
				}
				mPerflog.pushSessionInfo(SessPerflog.KEY_APP_LRS);
				notifyResult(true, data);
				break;
			default:
				break;
		}
		//DebugLog.LogD( "get result time is"+(SystemClock.elapsedRealtime()-start) );
	}
	
	/**
	 * 传入音频
	 * @param data
	 * @param needvadNotify
	 * @throws SpeechError
	 */
	protected void uploadData(byte[] data, final int len, boolean needvadNotify) 
		throws SpeechError{	

		// 第一次向msc写入音频时间
		if (false == mFirstDataWrite) {
			mFirstDataWrite = true;
			mPerflog.pushSessionInfo(SessPerflog.KEY_APP_FAU);
		}

		mSession.pushAudioData(data, len);
		// check vad state
		if (needvadNotify) {
			int volume = mSession.getAudioVolume();
			DebugLog.LogI("QISRAudioWrite volume:" + volume);
			callbackVolume(data, volume);
		}
	}

	/**
	 * record callback from PcmRecorder
	 * 
	 * @throws IOException
	 */
	@Override
	public void onRecordBuffer(byte[] data,int offset,int length) {
		// record data notify
		if(data == null || length <= 0 || data.length < length)
			return;
		if (length > 0 && isRunning()) {
			
			if(false == mFirstRecordAudio){
				mFirstRecordAudio = true;
				mPerflog.pushSessionInfo(SessPerflog.KEY_REC_START);
			}

			try {
				if(mAccesser == null) {
					if( MscKeys.VALUE_AUDIO_EXTERNAL == this.mAudioSource ){
						this.mAccesser = AudioAccessor.createBufferAccessor();
					}else if( MscKeys.VALUE_AUDIO_EXTERNALFILE == this.mAudioSource ){
						this.mAccesser = AudioAccessor.createReadOnlyAccessor(mFilePath);
						//设置音频文件的采样率
						getParam().putParam( SpeechConstant.SAMPLE_RATE,
								this.mAccesser.getAudioInfo(AudioAccessor.AudioKeys.RATE),
								false );
					}
				}
				this.mAccesser.putAudio(data, length);
			} catch (Exception e) {
				DebugLog.LogE(e);
				exit(new SpeechError(e));
				return;
			}
			
			this.updateCatchLeft();

			if( !mIsProcRecordDataOn ){
				mIsProcRecordDataOn = true;
				sendMsg(obtainMessage(MSG_RECORD_AUDIO,null));
			}
		}
	}

	/**
	 * 录音错误
	 */
	@Override
	public void onError(SpeechError error) {
		//记录应用层错误码
		exit(error);
	}

	/**
	 * 录音启动
	 */
	@Override
	public void onRecordStarted(boolean success) {
		mPerflog.pushSessionInfo(SessPerflog.KEY_REC_READY);
		mIsRecording = success;
	}
	
	/**
	 * 返回结构化日志对象，便于外部写入
	 * @return
	 */
	public SessPerflog getPerfLog()
	{
		return mPerflog;
	}

	/**
	 * 录音机释放通知
	 */
	@Override
	public void onRecordReleased() {
		
	}

	/**
	 * 返回服务端ID
	 */
	@Override
	public String getSessionID()
	{
		if( TextUtils.isEmpty(mSessionID) ){
			mSessionID = mSession.getSessionID();
		}
		return mSessionID;
	}
	
	/**
	 * 返回客户端ID
	 */
	@Override
	public String getClientID()
	{
		return mSession.getClientID();
	}
	
	private boolean isRecordMode(){
		return MscKeys.VALUE_AUDIO_EXTERNAL != this.mAudioSource
				&&  MscKeys.VALUE_AUDIO_EXTERNALFILE != this.mAudioSource;
	}
	
	private void assertSessionTimes() throws SpeechError{
		if( (!isRecordMode()||(isRecordMode()&&!isRecording()))
				&& this.getMaxSessionTry()!=MscKeys.VALUE_IST_SESSION_TRY_INFINITY
				&& this.getMaxSessionTry()<=mSessionRetryCount){
			DebugLog.LogE( "proc_Msg_Session_Begin max session try:"+this.getMaxSessionTry() );
			throw new SpeechError( mError );
		}
	}
	
	private boolean isRecording(){
		DebugLog.LogE( "isRecording:"+mIsRecording );
		return mIsRecording;
	}
	
	private boolean hasMoreData(){
		DebugLog.LogE("hasMoreData >>> isRecording ? "+isRecording()+" && datalength == "+this.getDataLength());
		return isRecording()
				|| this.getDataLength()>this.mLastMscOffset;
	}
	
	private synchronized void restartSession() throws SpeechError{
		if( mIsRestarting ){
			return;
		}
		
		assertSessionTimes();
		
		//更新当前会话信息
		this.updateUploadBytes();
		this.updateSessionID();
		this.updateSyncID();
		
		//结束当前会话
		mSession.sessionEnd("restart");
		
		//重置要读取的音频位置为已上传字节位置
		mLastMscOffset = mLastServerOffset;
		
		this.mSession.mClientID = null;
		
		//设置会话开始参数
		if( null != this.mSessionID ){
			this.getParam().putParam( SpeechConstant.IST_SESSION_ID, mSessionID );
			this.getParam().putParam( SpeechConstant.IST_AUDIO_UPLOADED,
					Integer.toString(mLastServerOffset),
					true );
			this.getParam().putParam( SpeechConstant.IST_SYNC_ID,
					Integer.toString(mSyncID),
					true );
		}
		
		//重启会话
		sendMsg( MSG_SESSION_BEGIN, Priority.max, false, 0 );
		
		mIsRestarting = true;
	}

	private int getMaxSessionTry(){
		return this.mMaxSessionTry;
	}

	private long getDataLength(){
		if( null != this.mAccesser ){
			DebugLog.LogD( "getDataLength len="+this.mAccesser.getDataLength() );
			return this.mAccesser.getDataLength();
		}else{
			DebugLog.LogE( "getDataLength file accesser is null." );
			return -1;
		}
		
	}
	
	private void closeAudioFile() throws IOException {
		if (null != this.mAccesser) {
			this.mAccesser.close();
			this.mAccesser = null;
		}
	}
	
	private int getAudioData( ) throws SpeechError, IOException{
		if( null != this.mAccesser ){
			return this.mAccesser.getAudio( mBuffer );
		}else{
			return 0;
		}
	}
	
	private boolean isSessionOn(){
		return null != this.mSession.mClientID;
	}
	
	public int getUploadBytes(){
		return this.mLastServerOffset;
	}

	private void updateSessionID(){
		if( null == this.mSessionID ){
			this.mSessionID = getSessionID();
			if( null!=this.mSessionID && null!=this.mListener ){
				Bundle bundle = new Bundle();
				bundle.putString(SpeechEvent.KEY_EVENT_SESSION_ID, this.mSessionID );
				mListener.onEvent(SpeechEvent.EVENT_SESSION_ID,0, 0,bundle);
			}
		}
	}
	
	private void updateUploadBytes(){
		final int uploadLen = this.mSession.getIntValue( SpeechConstant.IST_AUDIO_UPLOADED, mLastServerOffset );
		DebugLog.LogS( "updateUploadBytes", "uploadLen="+uploadLen );
		if( uploadLen != this.mLastServerOffset ){
			this.mLastServerOffset = uploadLen;
			if( null != this.mListener ){
				int totalLen = MscKeys.VALUE_AUDIO_EXTERNAL==this.getAudioSource() ? (int)this.getDataLength() : -1;
				boolean isUploadComplete = !isRecording() 
						&& this.getDataLength()==this.mLastServerOffset;
				Bundle bundle = new Bundle();
				bundle.putBoolean( SpeechEvent.KEY_EVENT_IST_UPLOAD_COMPLETE, isUploadComplete );
				this.mListener.onEvent( SpeechEvent.EVENT_IST_UPLOAD_BYTES, uploadLen, totalLen, bundle );
			}
		}
	}
	
	private void updateCatchLeft(){
		if( !this.isRecordMode() && null!=this.mListener && null!=mAccesser ){
			this.mListener.onEvent( SpeechEvent.EVENT_IST_CACHE_LEFT, mAccesser.getCacheLeft(), 0, null );
		}
	}
	
	public int getSyncID(){
		return this.mSyncID;
	}
	
	private void updateSyncID(){
		final int syncid = this.mSession.getIntValue( SpeechConstant.IST_SYNC_ID, mSyncID );
		DebugLog.LogS( "updateSyncID", "syncid="+syncid );
		if( syncid != this.mSyncID ){
			this.mSyncID = syncid;
			if( null != this.mListener ){
				this.mListener.onEvent( SpeechEvent.EVENT_IST_SYNC_ID, syncid, 0, null );
			}
		}
	}
	
	public String getAudioPath(){
		if( null != this.mAccesser ){
			return mAccesser.getFilePath();
		}else{
			return null;
		}
	}
	
	private void stopWriteAutio() throws SpeechError{
		// upload end flag
		mSession.pushEndFlag();
		sendMsg( obtainMessage(MSG_WAIT_RESULT, null) );
	}
	
	protected void proc_Msg_Wait_Result() throws Exception{
		// 更新当前会话信息
		this.updateUploadBytes();
		this.updateSessionID();
		this.updateSyncID();

		int waitTime = this.mSession.getIntValue(MscKeys.IST_RESULT_TIME_LEFT, -1);
		DebugLog.LogD("onWaiting msc waitTime=" + waitTime);
		if (waitTime >= 0 && waitTime != this.mResultSecondLeft) {
			mStatusBegin = System.currentTimeMillis();
			this.mResultSecondLeft = waitTime;
			if (null != this.mListener) {
				this.mListener.onEvent(SpeechEvent.EVENT_IST_RESULT_TIME,
						waitTime / 1000, 0, null);
			}
		}

		ResultStatus status = ResultStatus.noResult;
		try {
			status = requestResult();
		} catch (SpeechError error) {
			if (ErrorCode.MSP_ERROR_TIME_OUT == error.getErrorCode()) {
				restartSession();
			} else {
				throw (error);
			}
		}

		if (this.mResultSecondLeft >= 0
				|| ResultStatus.noResult!=status) {
			timeOutCheck(mStatusBegin, this.mResultSecondLeft+this.mNetTimeOut);
		}

		sendMsg(MSG_WAIT_RESULT, Priority.max, false, WAIT_RESULT_PERIOD);
		
	}//end of function proc_Msg_Wait_Result
	
	protected ResultStatus requestResult() throws SpeechError, UnsupportedEncodingException
	{
		final ResultStatus status = mSession.getStatus();
		
		byte []resultBytes = null;
		if( ResultStatus.noResult!=status ){
			if(mSession.getResultData() != null && mSession.getResultData().length > 0){
				String result = new String(mSession.getResultData(), DataUtil.GB2312);
				resultBytes = result.getBytes(DataUtil.UTF8);
			}
		}
		DebugLog.LogD( "result:"+resultBytes );
		
		Message msg = obtainMessage(MSG_RESULT, status.ordinal(), 0, resultBytes);
		//消息队列在执行很耗时的任务 后至的结果数据被插入队列前，导致之前的结果无法展示
		//ps:概率很小
		if(hasMessages( MSG_RESULT ))
			sendMsg(msg, Priority.normal, false, 0);
		else
			sendMsg(msg, Priority.max, false, 0);
		
		return status;
	}

}