package com.iflytek.cloud.msc.ivw;

import java.io.UnsupportedEncodingException;

import android.content.Context;
import android.os.SystemClock;
import android.text.TextUtils;

import com.iflytek.cloud.ErrorCode;
import com.iflytek.cloud.SpeechConstant;
import com.iflytek.cloud.SpeechError;
import com.iflytek.cloud.msc.module.MscHandler;
import com.iflytek.cloud.msc.module.MscSession;
import com.iflytek.cloud.msc.util.log.DebugLog;
import com.iflytek.cloud.msc.util.log.PerfLogger;
import com.iflytek.cloud.param.ParamBuilder;
import com.iflytek.msc.MSC;
import com.iflytek.msc.MSCSessionInfo;


/**
 * 处理唤醒相关操作
 * @author longwang
 *
 */
public class IvwSession extends MscSession{
	/** member properties */
	private MSCSessionInfo    mSessionOut  = new MSCSessionInfo();
	private MSCSessionInfo    mWriteout  = new MSCSessionInfo();

	private final static int       IVW_AUDIO_SAMPLE_CONTINUE = 0x02;
	private final static int       IVW_AUDIO_SAMPLE_LAST     = 0x04;
	@Override
	public int sessionBegin(Context context,
			String sessionType, MscHandler handler) throws SpeechError,
			UnsupportedEncodingException {
		String params = ParamBuilder.composeIvwSessionParam(context,sessionType,handler);
		String grammar=  handler.getParam().getString(SpeechConstant.CLOUD_GRAMMAR);
		long d1 = SystemClock.elapsedRealtime();
		DebugLog.LogD( "ivw sessionbegin begin" );
		PerfLogger.appendInfo(PerfLogger.MSC_SESSION_BIGNE, null);
		if (TextUtils.isEmpty(grammar)) {
			mClientID = MSC.QIVWSessionBegin(null,params.getBytes(handler.getParamEncoding()), mSessionOut);
		}else {
			mClientID = MSC.QIVWSessionBegin(grammar.getBytes(handler.getParamEncoding()),params.getBytes(handler.getParamEncoding()), mSessionOut);
		}
		PerfLogger.appendInfo(PerfLogger.SESSION_BEGIN_END, null);
		DebugLog.LogD("ivw sessionBegin ErrCode:" + mSessionOut.errorcode
				+ " time:" + (SystemClock.elapsedRealtime() - d1));
		int errorCode = mSessionOut.errorcode;
		if (0 != errorCode) {
			//增加防止10109、10132错误直接返回
			if(errorCode != ErrorCode.MSP_ERROR_CREATE_HANDLE
					&& errorCode != ErrorCode.MSP_ERROR_OVERFLOW && errorCode != ErrorCode.MSP_ERROR_INVALID_OPERATION)
				throw new SpeechError(errorCode);
		}
		return errorCode;
	}

	@Override
	public void sessionEnd(String reason) {
		if (null == mClientID) {
            return;
        }
        DebugLog.LogD("sessionEnd enter ");
        long d1 = System.currentTimeMillis();
        boolean bRet = 0 == MSC.QIVWSessionEnd(mClientID, reason.getBytes());
        DebugLog.LogD("sessionEnd leave: " + bRet + " time:"
                + (System.currentTimeMillis() - d1));
        mClientID = null;
        mSessionID = null; 
	}

	synchronized public void pushAudioData(final byte[] waveData,
			final int waveLen) throws SpeechError {
		writeData(waveData, waveLen, IVW_AUDIO_SAMPLE_CONTINUE);
	}

	synchronized private void writeData(final byte[] waveData,
			final int waveLen, final int audioStatus) throws SpeechError {
		int nRet = MSC.QIVWAudioWrite(mClientID, waveData, waveLen,audioStatus,mWriteout);
		mSessionOut.sesstatus = mWriteout.sesstatus;
		DebugLog.LogI("QIVWAudioWrite length:" + waveLen);
		if (0 != nRet) {
			throw new SpeechError(mWriteout.errorcode);
		}
	}
	synchronized public void pushEndFlag() throws SpeechError{
		 byte[] data = new byte[0];
		 PerfLogger.appendInfo(PerfLogger.LAST_DATA_FLAG, null);
		 DebugLog.LogD( "ivw session pushEndFlag" );
	     writeData(data, 0, IVW_AUDIO_SAMPLE_LAST);
	}
}
