/*
 * 文件名：IvwOptimize.java 5.0.1085.1115 Nov 9,2015
 * 描述 ：唤醒优化管理对象：唤醒资源查询，唤醒资源下载等；
 * 作者:iFLytek
 * 版权：科大讯飞
 */
package com.iflytek.cloud.msc.ivw;

import java.io.UnsupportedEncodingException;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import org.json.JSONTokener;

import android.content.Context;
import android.os.Bundle;
import android.text.TextUtils;

import com.iflytek.cloud.ErrorCode;
import com.iflytek.cloud.RequestListener;
import com.iflytek.cloud.SpeechError;
import com.iflytek.cloud.SpeechEvent;
import com.iflytek.cloud.SpeechUtility;
import com.iflytek.cloud.msc.util.Config;
import com.iflytek.cloud.msc.util.DataUtil;
import com.iflytek.cloud.msc.util.Encrypter;
import com.iflytek.cloud.msc.util.FileDownloader;
import com.iflytek.cloud.msc.util.HttpRequest;
import com.iflytek.cloud.msc.util.HttpRequest.HttpRequestListener;
import com.iflytek.cloud.msc.util.log.DebugLog;
import com.iflytek.cloud.param.HashParam;
import com.iflytek.cloud.param.MscKeys;
import com.iflytek.cloud.param.ParamBuilder;
import com.iflytek.cloud.util.FileDownloadListener;
import com.iflytek.msc.MSC;
import com.iflytek.msc.MSCSessionInfo;

public class OptimizeManager{
	// 资源头信息字段
	private static final String ATHINFO = "athinfo";
	// 资源路径
	private static final String RESPATH = "respath";
	// 应用id 唯一
	private static final String APPID = "appid";
	// 资源id 唯一
	private static final String RESID = "resid";
	// 唤醒词列表
	private static final String WAKELIST = "wakelist";
	// 资源类型
	private static final String RESTYPE = "restype";
	// 用户id
	private static final String UID = "uid";
	// 用户授权时间
	private static final String EXPIREDATE = "expiredate";
	// 唤醒资源版本号
	private static final String RESVER = "resver";
	// 资源生成日期
	private static final String RESTIME = "restime";
	// 唤醒引擎版本号
	private static final String WAKEVER = "wakever";
	// 头信息唤醒词JsonObject
	private static final String WAKELIST_JSON = "ivwword";
	// 头信息唤醒词JsonArray
	private static final String WAKELIST_ARRAY = "words";
	// 头信息唤醒词
	private static final String WAKE_TEXT = "text";
	// 头信息唤醒词门限值
	private static final String WAKE_THRESHOLD = "threshold";
	// 应用级资源
	private static final String RESTYPE_D = "d";
	// 筛选区间
	private static final String NET_VAL = "netval";
	// 唤醒词连接符
	private static final String SYMBOL_COLON = ":";
	// 无有效值时，默认取值
	private static final String UNKOWN = "unkown";
	// 上传信息设备信息字段
	private static final String REQUEST_HEADER = "header";
	// 上传信息资源信息资源
	private static final String REQUEST_INFO = "info";
	
	private Context mContext = null;
	// 查询请求对象
	private HttpRequest mHttpRequest = null;
	// 请求回调监听
	private RequestListener mRequestListener = null;
	// 资源下载对象
	private FileDownloader mHttpDownload = null;
	
	// 测试请求网址
//	private static String LINK_URL = "http://172.16.154.240:8888/wakeup/";
	// 正式请求网址
//	private static String LINK_URL = "http://60.166.12.151/wakeup/";
	private static String LINK_URL = "http://wke.openspeech.cn/wakeup/";
	
	
	
	public OptimizeManager(Context context) {
		mContext = context;
		mHttpDownload = FileDownloader.getDownLoadManager(mContext);
		mHttpRequest = new HttpRequest();
	}
	
	/**
	 * 向服务端发送查询请求
	 * @param requestInfo 请求资源信息
	 * @param listener 资源查询回调监听
	 * @return
	 */
	public int queryRequest(JSONObject requestInfo, RequestListener listener) {
		try {
			if(requestInfo == null) {
				return ErrorCode.ERROR_INVALID_DATA;
			}
			
			if(listener != null) {
				mRequestListener = listener;
			}
			
			// 组装设备信息及资源信息，组成完整的请求信息
			JSONObject post = new JSONObject();
			post.put(REQUEST_HEADER, ParamBuilder.composeIvwResSearchParam(mContext, new HashParam()));
			post.put(REQUEST_INFO, requestInfo);
			
			DebugLog.LogS("post data:" + post);
			
			// 对上传数据进行zip加密
			byte[] data = Encrypter.zip5xEncode(post.toString().getBytes());
			
			// 发送请求
			mHttpRequest.setConectType(HttpRequest.CONNECT_POST);
			mHttpRequest.setRequest(LINK_URL, null, data);
			mHttpRequest.startRequest(requestListener);
			return ErrorCode.SUCCESS;
		} catch (Exception e) {
			DebugLog.LogE(e);
			return ErrorCode.ERROR_NET_EXCEPTION;
		}
	}
	
	/**
	 * 资源下载，根据传入的链接及保存路径，下载文件到指定位置
	 * @param url 下载链接
	 * @param path 保存文件路径
	 * @param listener 下载回调监听
	 * @return
	 */
	public int downloadResource(String url, String path, String md5, FileDownloadListener listener) {
		if(TextUtils.isEmpty(url) || TextUtils.isEmpty(path)) {
			return ErrorCode.ERROR_INVALID_PARAM;
		}
		
		return mHttpDownload.startDownload(url, path, md5, listener);
	}

	/**
	 * 获取资源请求信息
	 * 读取用户资源和本地资源头信息，并进行组装
	 * 对组装后的Json数据进行匹配，返回最新资源头信息请求Json数据
	 * @param resourcePath 用户传入路径
	 * @param configPath  本地缓存路径
	 * @return 头信息请求数据
	 */
	public static synchronized JSONObject getResRequestInfo(String resourcePath, String configPath) {
		String param = "";
		if (TextUtils.isEmpty(configPath)) {
			// 配置资源地址为空，返回用户资源地址信息
			return getIvwResInfo(resourcePath, param);
		} else {
			JSONObject resInfo = getIvwResInfo(resourcePath, param);
			// 用户资源地址为空，返回null
			if (resInfo == null) {
				return null;
			}
			
			JSONObject cfgInfo = getIvwResInfo(configPath, param);
			// 配置资源地址信息为null，直接用户资源地址信息
			if (cfgInfo == null) {
				return resInfo;
			}

			JSONObject root = compareIvwResInfo(resInfo, cfgInfo);
			return root;
		}
	}
	
	/**
	 * compare two res object, if leftObj new, return true, else return false
	 * @param leftObj
	 * @param rightObj
	 * @return if leftObj new, return true, else return false
	 */
	public static boolean isResNew( final JSONObject leftObj, final JSONObject rightObj ){
		boolean ret = false;
		
		if( null != leftObj ){
			if( null == rightObj ){
				ret = true;
			}else{
				try {
					// 对两个资源的版本号进行比较
					if( leftObj.getString(RESVER).equalsIgnoreCase(UNKOWN) ) {
						ret = false;
					} else if( rightObj.getString(RESVER).equalsIgnoreCase(UNKOWN) ){
						ret = true;
					}else{
						ret =  DataUtil.compareJsonString(APPID, leftObj, rightObj)
								&& DataUtil.compareJsonString(RESID, leftObj, rightObj)
								&& DataUtil.compareJsonString(RESTYPE, leftObj, rightObj)
								&& DataUtil.compareJsonString(WAKELIST, leftObj, rightObj)
								&& leftObj.getLong(RESTIME)>rightObj.getLong(RESTIME);
					}//end of if-else	
				} catch (JSONException e) {
					DebugLog.LogE( e );
					ret = false;
				}//end of try-catch
			}//end of if-else right null
		}//end of if left not null
		
		return ret;
	}
	
	/**
	 * 读取唤醒词头信息，并组装返回新的json
	 * @param resPath
	 * @param param
	 * @return
	 * @throws UnsupportedEncodingException
	 */
	public static JSONObject getIvwResInfo(String resPath, String param){
		MSCSessionInfo sessionInfo = new MSCSessionInfo();
		try {
			DebugLog.LogD( "getIvwResInfo resPath: "+resPath );
			int ret = MSC.QIVWGetResInfo(resPath.getBytes(), param.getBytes(), sessionInfo);
			
			if(ret != ErrorCode.SUCCESS) {
				DebugLog.LogE("read ivw resoure error:"+ ret);
				return null;
			}
		} catch (UnsatisfiedLinkError error) {
			DebugLog.LogE("get res info error, unsatisfiedlinkerror");
			return null;
		} catch (Exception e) {
			DebugLog.LogE("get res info error");
			return null;
		}
		
		String result = null;
		try {
			result = new String(sessionInfo.buffer, DataUtil.UTF8);
		} catch (UnsupportedEncodingException e) {
			DebugLog.LogE(e);
			return null;
		}
		DebugLog.LogS("resInfo:"+new String(sessionInfo.buffer));
		return composeResInfo(resPath, result);
	}
	
	// 取消本次查询操作
	public void cancelRequest() {
		if(mHttpRequest != null) {
			mHttpRequest.cancel();
			mHttpRequest = null;
		}
		mRequestListener = null;
	}
	
	/**
	 * 查询网络请求回调
	 */
	public HttpRequestListener requestListener = new HttpRequestListener() {
		@Override
		public void onResult(HttpRequest request, byte[] data) {
			try {
				String updateInfo = new String(data, DataUtil.UTF8);
				DebugLog.LogS("updateInfo:" + updateInfo);

				// 检测是否发生错误
				JSONObject obj = new JSONObject(updateInfo);
				
				try {
					// 获取筛选区间
					int netval = Integer.valueOf((String) obj.remove(NET_VAL));
					Config.getConfig(mContext).putInt(MscKeys.IVW_NETVAL, netval);
				} catch (Exception e) {
					DebugLog.LogE(e);
				}
				
				int ret = obj.getInt("ret");
				if(ErrorCode.SUCCESS != ret) {
					String sid = obj.getString("sid");
					DebugLog.LogD( "query ivw res sid:"+ sid );
					// 服务端返回信息中有明确错误码，则解析出错误信息及sid
					if (mRequestListener != null) {
						// 通过onEvent抛出服务端会话id
						Bundle bundle = new Bundle();
						bundle.putString(SpeechEvent.KEY_EVENT_SESSION_ID, sid);
						mRequestListener.onEvent(SpeechEvent.EVENT_SESSION_ID, bundle);
					}
					onError(new SpeechError(ret));
				} else {
					if (mRequestListener != null) {
						mRequestListener.onBufferReceived(data);
					}
					onError(null);
				}
			} catch (Exception e){
				DebugLog.LogE(e);
				onError(new SpeechError(ErrorCode.ERROR_INVALID_RESULT));
			}
		}	
			
		@Override
		public void onError(SpeechError error) {
			if(error != null) { 	
				DebugLog.LogD("query resource error. errorcode:"+error.getErrorCode());
			} else {
				DebugLog.LogS("query resource succeed");
			}
			if(mRequestListener != null) {
				mRequestListener.onCompleted(error);
			}
		}
	};
	
	/**
	 * 唤醒词头信息组装
	 * 通过解析头信息，并组装成新的json
	 * @param resInfo
	 * @return 
	 */
	private static JSONObject composeResInfo(String resPath, String resInfo) {
		JSONObject root = new JSONObject();
		try {
			//保存路径
			root.put(RESPATH, resPath);
			
			JSONTokener jTokener = new JSONTokener(resInfo);
			JSONObject jsObject = new JSONObject(jTokener);
			
			JSONObject athinfo = jsObject.getJSONObject(ATHINFO);
			// 基本信息
			root.put(APPID, athinfo.getString(APPID));
			root.put(RESID, athinfo.getString(RESID));
			// 判断资源类型
			String restype = athinfo.getString(RESTYPE);
			root.put(RESTYPE, restype);
			if (!RESTYPE_D.equalsIgnoreCase(restype)) {
				root.put(UID, athinfo.getString(UID));
				root.put(EXPIREDATE, athinfo.getString(EXPIREDATE));
			}
			
			// 唤醒词列表
			JSONArray words = jsObject.getJSONObject(WAKELIST_JSON).getJSONArray(WAKELIST_ARRAY);
			String wakelist = "";
			for(int i=0;i<words.length();i++){
				wakelist += words.getJSONObject(i).getString(WAKE_TEXT);
				wakelist += SYMBOL_COLON;
				wakelist += words.getJSONObject(i).getInt(WAKE_THRESHOLD);
				wakelist += ",";
			}
			wakelist = wakelist.substring(0, wakelist.length()-1);
			root.put(WAKELIST, wakelist);
			
			// 资源版本信息
			try {
				root.put(RESVER, jsObject.getString(RESVER));
			} catch (JSONException e) {
				root.put(RESVER, UNKOWN);
			}
			
			// 获取资源生成时间，若无，默认0
			try {
				root.put(RESTIME, jsObject.getLong(RESTIME));
			} catch (JSONException e) {
				root.put(RESTIME, 0L);
			}
			
			//唤醒引擎版本获取
			String wakever = "";
			if(SpeechUtility.getUtility() != null) {
				wakever = SpeechUtility.getUtility()
						.getParameter(MscKeys.VER_IVW);
			}
			root.put(WAKEVER, wakever);
			
			DebugLog.LogS("root:" + root.toString());
		} catch (JSONException e) {
			root = null;
			DebugLog.LogD("compose resinfo error");
		}
		return root;
	}
	
	/**
	 * 比较两个资源请求信息，返回最新资源信息
	 * object1与object2进行比较
	 * @param usrObject
	 * @param cfgObject
	 * @return
	 */
	private static JSONObject compareIvwResInfo(JSONObject usrObject, JSONObject cfgObject) {
		try {
			// 对两个资源的版本号进行比较
			if(usrObject.getString(RESVER).equalsIgnoreCase(UNKOWN)) {
				DebugLog.LogS("user ivw resver unkown");
				return null;
			} else if(cfgObject.getString(RESVER).equalsIgnoreCase(UNKOWN)
					|| !usrObject.getString(RESVER).equalsIgnoreCase(cfgObject.getString(RESVER))){
				DebugLog.LogS("cfg ivw resver unkown or unequal");
				return usrObject;
			}
			
			// 对appid、resid、restype、wakelist、restime进行比较
			if(!DataUtil.compareJsonString(APPID, usrObject, cfgObject)
					|| !DataUtil.compareJsonString(RESID, usrObject, cfgObject)
					|| !DataUtil.compareJsonString(RESTYPE, usrObject, cfgObject)
					|| !DataUtil.compareJsonString(WAKELIST, usrObject, cfgObject)
					|| !DataUtil.compareJsonLong(RESTIME, usrObject, cfgObject)){
				return usrObject;
			}
		} catch (JSONException e) {
			DebugLog.LogE( e );
			return null;
		}
		return cfgObject;
	}
	
}
