package com.iflytek.cloud.msc.module;

import java.io.UnsupportedEncodingException;

import android.content.Context;

import com.iflytek.cloud.SpeechError;

/**
 * 会话信息，负责与MSC底层进行交互，保存会话ID等信息
 * @author yjzhao
 *
 */
public abstract  class MscSession {

	// 会话本地SessionID，非服务端会话ID
	public char[]	mClientID	= null;
	//服务端的会话ID
	public String   mSessionID  = null;
	//获取服务端ID的Tag
	public static final String TAG_SID = "sid";
	//获取服务端音频url
	public static final String TAG_AUDIO_URL = "audio_url";
	
	/**
	 * 定义识别结果状态，
	 * 为了保证 ordinal函数返回的数值正确，请务必保证 hasResult, noResult, resultOver的值为0,2,5
	 */
	public enum ResultStatus {
		hasResult, undefined_1, noResult, undefined_3, undefined_4, resultOver,
	}

	/**
	 * 
	 * @param context 上下文
	 * @param extra 扩展参数，识别中表示语法ID
	 * @return 错误码
	 * @throws SpeechError MSC error 信息
	 * @throws UnsupportedEncodingException 未支持的编码错误
	 */
	public abstract int sessionBegin(Context context,String extra,MscHandler looper)throws SpeechError,
			UnsupportedEncodingException;
	
	/**
	 * 结束会话
	 * @param reason 结束原因
	 */
	public abstract  void sessionEnd(String reason);
	/**
	 * 获得客户端ID
	 */
	public String getClientID()
	{
		if(null != mClientID)
			return new String( mClientID );
		return null;
	}
	
}
