package com.iflytek.cloud.resource;

import java.util.Locale;

import android.text.TextUtils;
/**
 * 资源管理
 * @author iFlytek &nbsp;&nbsp;&nbsp;<a href="http://www.xfyun.cn/">讯飞语音云</a>
 *
 */
public class Resource {
	
	/**
	 * 默认语言为中文简体\中文、中文繁体、英语三种
	 */
	private static Locale LANGUAGE = Locale.CHINA;
	
	// text
	public static final int TEXT_POWER_LINK = 0;
	public static final int TEXT_HELP_LINK = 1;
	public static final int TEXT_KNOW = 2;
	public static final int TEXT_DETAIL = 3;
	public static final int TEXT_CANCEL = 4;
	public static final int TEXT_STOP = 5;
	public static final int TEXT_SET = 6;
	public static final int TEXT_MORE = 7;
	public static final int TEXT_RETRY = 8;
	public static final int TEXT_AGAIN = 9;
	public static final int TEXT_PLAYBACK = 10;
	public static final int TEXT_RETRIEVE = 11;
	public static final int TEXT_HELP_SMS = 12;
	public static final int TEXT_HELP_RECO = 13;
	
	public static final int TITLE_HELP = 0;
	public static final int TITLE_CONNECTING = 1;
	public static final int TITLE_RECORDING = 2;
	public static final int TITLE_RECOGNIZE_WAITING = 3;
	public static final int TITLE_AUDIO_REQUEST = 4;
	public static final int TITLE_ERROR = 5;
	public static final int TITLE_AUDIO_PLAYING = 6;
	public static final int TITLE_DATA_UPLOAD = 7;
	
	private Resource() {
		// do nothing
	}
	
	/**
	 * 设置语言
	 * @param language
	 */
	public static void setUILanguage(Locale language)
	{
		if(language == null)
			return;
		if(language.equals(Locale.US)
		  || language.equals(Locale.CHINA)
		  || language.equals(Locale.TRADITIONAL_CHINESE))
		{
			LANGUAGE = language;
		}
	}

	/**
	 * 获得语言名称
	 */
	public static String getLanguage()
	{
		return LANGUAGE.toString();
	}
	
	/**
	 * 检测语言是否和设置相同
	 * @param language
	 * @return
	 */
	public static boolean matchLanguage(String language)
	{	
		String trimd = "";
		if(language != null)
			trimd = language.trim();
		
		if(LANGUAGE.toString().equalsIgnoreCase(trimd))
		{
			return true;
		}
		
		// 防止中文简体设置不一致
		if(isChinese(trimd) && isChinese(LANGUAGE.toString()))
			return true;
		
		return false;
	}
	
	/**
	 * 判断语言是否为简体中文
	 * 	空字符、Locale.CHINA，Locale.CHINESE均判断为中午跟简体
	 * @return
	 */
	private static boolean isChinese(String language)
	{
		if(TextUtils.isEmpty(language))
			return true;
		if(Locale.CHINA.toString().equalsIgnoreCase(language) || 
		   Locale.CHINESE.toString().equalsIgnoreCase(language) )
			return true;
		return false;
	}
	
	public static void setTitle(int titleID,String description)
	{
		String[] titles = ResourceCN.titles;
		if(LANGUAGE.equals(Locale.US))
			titles = ResourceEN.titles;
		else if(LANGUAGE.equals(Locale.TRADITIONAL_CHINESE))
			titles = ResourceTW.titles;
		if(titleID >=0 && titleID < titles.length)
			titles[titleID] = description;
	}
		
	public static String getTitle(int titleID)
	{
		String[] titles = ResourceCN.titles;
		if(LANGUAGE.equals(Locale.US))
			titles = ResourceEN.titles;
		else if(LANGUAGE.equals(Locale.TRADITIONAL_CHINESE))
			titles = ResourceTW.titles;
		if(titleID >=0 && titleID < titles.length)
			return titles[titleID];
		return "";
	}
	
	public static String getText(int textID)
	{
		String[] texts = ResourceCN.texts;
		if(LANGUAGE.equals(Locale.US))
			texts = ResourceEN.texts;
		else if(LANGUAGE.equals(Locale.TRADITIONAL_CHINESE))
			texts = ResourceTW.texts;
		if(textID >=0 && textID < texts.length)
			return texts[textID];
		return "";
	}
	
	public static void setText(int textID,String text)
	{
		String[] texts = ResourceCN.texts;
		if(LANGUAGE.equals(Locale.US))
			texts = ResourceEN.texts;
		else if(LANGUAGE.equals(Locale.TRADITIONAL_CHINESE))
			texts = ResourceTW.texts;
		if(textID >=0 && textID < texts.length)
			texts[textID] = text;
	}
	
	public static String getErrorDescription(int errorType)
	{
		String[] errors = ResourceCN.errors;
		if(LANGUAGE.equals(Locale.US))
			errors = ResourceEN.errors;
		else if(LANGUAGE.equals(Locale.TRADITIONAL_CHINESE))
			errors = ResourceTW.errors;
		if(errorType > 0 && errorType < errors.length)
			return errors[errorType];
		return getErrorTag(TAG_ERROR_UNKNOWN);
	}
	
	public static void setErrorDescription(int errorType,String description)
	{
		String[] errors = ResourceCN.errors;
		if(LANGUAGE.equals(Locale.US))
			errors = ResourceEN.errors;
		else if(LANGUAGE.equals(Locale.TRADITIONAL_CHINESE))
			errors = ResourceTW.errors;
		if(errorType > 0 && errorType < errors.length)
			errors[errorType]= description;
	}
	
	public static final int TAG_ERROR_CODE = 0;
	public static final int TAG_ERROR_UNKNOWN = 1;
	public static String getErrorTag(int errtag)
	{
		String[] errors = ResourceCN.error_tag;
		if(LANGUAGE.equals(Locale.US))
			errors = ResourceEN.error_tag;
		else if(LANGUAGE.equals(Locale.TRADITIONAL_CHINESE))
			errors = ResourceTW.error_tag;
		if(errtag >= 0 && errtag < errors.length)
			return errors[errtag];
		return "";
	}
}
