/*
 * 文件名：SpeechEvaluatorImpl.java 2.5.1027.1040 Dec 11,2013
 * 描述 ：评测对象具体实现.
 * 作者:iFLytek
 * 版权：科大讯飞
 */
package com.iflytek.cloud.speech.impl;

import java.util.concurrent.ConcurrentLinkedQueue;

import android.content.Context;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;
import android.text.TextUtils;

import com.iflytek.cloud.ErrorCode;
import com.iflytek.cloud.EvaluatorListener;
import com.iflytek.cloud.EvaluatorResult;
import com.iflytek.cloud.SpeechConstant;
import com.iflytek.cloud.SpeechError;
import com.iflytek.cloud.msc.ise.MscEvaluator;
import com.iflytek.cloud.msc.module.SpeechModule;
import com.iflytek.cloud.msc.util.FileUtil;
import com.iflytek.cloud.msc.util.FuncAdapter;
import com.iflytek.cloud.msc.util.log.DebugLog;

public class SpeechEvaluatorImpl extends SpeechModule{
	
	// whether or not pause the background duration the session
	 private boolean mRequestFocus = false;

	public SpeechEvaluatorImpl(Context context) {
		super(context);
	}

	/**
	 * 
	 * @param answer
	 * @param textParams
	 * @param listener
	 * @return
	 */
	public int startEvaluating(String answer, String textParams, EvaluatorListener listener) {
		int ret = ErrorCode.SUCCESS;
		
		synchronized (mSynObj) {
			try{
				mRequestFocus = mSessionParams.getBoolean(SpeechConstant.KEY_REQUEST_FOCUS, true);
				if (null != mscer && mscer.isRunning()) {
					mscer.cancel(mSessionParams.getBoolean(SpeechConstant.ISE_INTERRUPT_ERROR, false));
				}
				
				mscer = new MscEvaluator(mContext, mSessionParams, getHandlerThread("eva"));

				FuncAdapter.Lock(mContext, mRequestFocus, null);
				((MscEvaluator) mscer).startListening(answer, textParams,
						new InternalEvaluatorListener(listener));
			}catch(SpeechError e){
				ret = e.getErrorCode();
				DebugLog.LogE( e );
			}catch( Throwable e ){
				ret = ErrorCode.ERROR_UNKNOWN;
				DebugLog.LogE( e );
			}//end of try-catch
		}//end of synchronized
		
		return ret;
	}
	
	/**
	 * 给平嵌提供二进制接口
	 * @param answer
	 * @param textParams 文本
	 * @param listener 评测接口监听器
	 * @return
	 */
	public int startEvaluating(byte[] answer, String textParams, EvaluatorListener listener) {
		int ret = ErrorCode.SUCCESS;
		
		synchronized (mSynObj) {
			try{
				mRequestFocus = mSessionParams.getBoolean(SpeechConstant.KEY_REQUEST_FOCUS, true);
				if (null != mscer && mscer.isRunning()) {
					mscer.cancel(mSessionParams.getBoolean(SpeechConstant.ISE_INTERRUPT_ERROR, false));
				}
				
				mscer = new MscEvaluator(mContext,mSessionParams,getHandlerThread( "eva" ));

				FuncAdapter.Lock(mContext, mRequestFocus, null);
				((MscEvaluator)mscer).startListening(answer, textParams,
						new InternalEvaluatorListener(listener));
			}catch(SpeechError e){
				ret = e.getErrorCode();
				DebugLog.LogE( e );
			}catch( Throwable e ){
				ret = ErrorCode.ERROR_UNKNOWN;
				DebugLog.LogE( e );
			}//end of try-catch
		}//end of synchronized
		
		return ret;
	}
	
	/**
	 * 写入录音数据
	 * @param buffer
	 * @return
	 */
	public boolean writeAudio(byte[] buffer, int offset, int length) {
		synchronized (mSynObj) {
			if (mscer == null) {	
				DebugLog.LogD("writeAudio error, no active session.");
				return false;
			}
			if(buffer == null || buffer.length <=0) {
				DebugLog.LogD("writeAudio error,buffer is null.");
				return false;
			}
			if(buffer.length < (length + offset)) {
				DebugLog.LogD("writeAudio error,buffer length < length.");
				return false;
			}
			((MscEvaluator)mscer).onRecordBuffer(buffer, offset, length);
			return true;	
		}
	}

	/**
	 * Stops listening for speech.
	 */
	public void stopEvaluating() {
		synchronized (mSynObj) {
			if (mscer != null) {
				((MscEvaluator)mscer).stopListening(true);	
			}
		}
	}
	
	/**
	 * 取消消息，保存底层事件
	 */
	@Override
	public void cancel(boolean notifyError) {
		synchronized( mSynObj ){
			sessionEnd();
		}//end of sync
		
		super.cancel(notifyError);
	}

	/**
	 * return audio record buffer
	 */
	public ConcurrentLinkedQueue<byte[]> getLastBuffer() {
		synchronized (mSynObj) {
			if(mscer != null) {
				return ((MscEvaluator)mscer).getBuffer();
			} else {
				return null;
			}
		}
	}
	
	@Override
	protected boolean onFini() {
		return super.onFini();
	}

	protected void sessionEnd() {
		if( null != mscer ){
			String recordFile = mscer.getParam().getString(
	                SpeechConstant.ISE_AUDIO_PATH);
	        if (!TextUtils.isEmpty(recordFile)) {
	        	if( FileUtil.saveFile(((MscEvaluator) mscer).getBuffer(),recordFile)){
	        		String format = mscer.getParam().getString(SpeechConstant.AUDIO_FORMAT, null);
	        		int sample = mscer.getParam().getInt(SpeechConstant.SAMPLE_RATE, mscer.mSampleRate);
	        		FileUtil.formatPcm(format, recordFile, sample);
	        	}
	        }
		}//end of mscer not null
		
        // 识别会话结束，后台音乐继续播放.
        FuncAdapter.UnLock(mContext, mRequestFocus, null);
	}
	
	private final class InternalEvaluatorListener implements EvaluatorListener {
		private static final int MSG_ERROR = 0;
		private static final int MSG_AUDIO_BUFFER = 1;
		private static final int MSG_SPEECH_BEGIN = 2;
		private static final int MSG_SPEECH_END = 3;
		private static final int MSG_RESULT = 4;
		private static final int MSG_CANCEL = 5;
		private static final int MSG_EVENT = 6;
		
		private EvaluatorListener mUserEvaluatorListener = null;

		public InternalEvaluatorListener(EvaluatorListener outlistener) {
			mUserEvaluatorListener = outlistener;
		}

		private Handler mUiHandler = new Handler(Looper.getMainLooper()) {
			@Override
			public void handleMessage(Message msg) {
				if (mUserEvaluatorListener == null) {
					return;
				}
					
				switch (msg.what) {
				case MSG_ERROR:
					mUserEvaluatorListener.onError((SpeechError) msg.obj);
					break;
				case MSG_CANCEL:
//					mUserEvaluatorListener.onCancel();
					break;
				case MSG_AUDIO_BUFFER:
					mUserEvaluatorListener.onVolumeChanged(msg.arg1,(byte[])msg.obj);
					break;
				case MSG_SPEECH_BEGIN:
					mUserEvaluatorListener.onBeginOfSpeech();
					break;
				case MSG_SPEECH_END:
					mUserEvaluatorListener.onEndOfSpeech();
					break;
				case MSG_RESULT:
					mUserEvaluatorListener.onResult((EvaluatorResult) msg.obj, 
							(msg.arg1 == 1 ? true : false));
					break;
				case MSG_EVENT:
					Message mbody = (Message) msg.obj;
        			// 自定义消息数据
					mUserEvaluatorListener.onEvent(mbody.what,
        					mbody.arg1, mbody.arg2,(Bundle)mbody.obj);
					break;
				default:
					break;
				}
				super.handleMessage(msg);
			}
		};

		@Override
		public void onEndOfSpeech() {
			Message msg = mUiHandler.obtainMessage(MSG_SPEECH_END, 0, 0, null);
			mUiHandler.sendMessage(msg);
		}

		@Override
		public void onResult(EvaluatorResult result, boolean isLast) {
			// 当返回最后一个结果，判断是否需要保存音频或继续音乐播放
            if (isLast) {
                sessionEnd();
            }
			Message msg = mUiHandler.obtainMessage(MSG_RESULT, 
					isLast == true? 1 : 0, 0, result);
			mUiHandler.sendMessage(msg);
		}

		@Override
		public void onBeginOfSpeech() {
			DebugLog.LogD("onBeginOfSpeech");
			Message msg = mUiHandler.obtainMessage(MSG_SPEECH_BEGIN, 0, 0, null);
			mUiHandler.sendMessage(msg);
		}

		/**
		 * Evaluator session end, continue to play background music
		 */
		@Override
		public void onError(SpeechError error) {
			sessionEnd();
			Message msg = mUiHandler.obtainMessage(MSG_ERROR, error);
			mUiHandler.sendMessage(msg);
		}

		@Override
		public void onVolumeChanged(int volume ,byte[] buffer) {
			Message msg = mUiHandler.obtainMessage(MSG_AUDIO_BUFFER, volume, 0, buffer);
			mUiHandler.sendMessage(msg);
		}

		@Override
		public void onEvent(int eventType, int arg1, int arg2, Bundle obj) {
			Message msg = new Message();
            msg.what = eventType;
            msg.arg1 = arg1;
            msg.arg2 = arg2;
            msg.obj = obj;

            Message.obtain(mUiHandler, MSG_EVENT, msg).sendToTarget();
		}
	}

	/**
	 * 是否正在做评测.
	 */
	public boolean isEvaluating() {
		return isRunning();
	}
}
