/*
 * 文件名：ResourceUtils.java 2.5.1027.1040 Dec 11,2013
 * 描述 ：用于获取界面图片及颜色资源
 * 作者:iFLytek
 * 版权：科大讯飞
 */
package com.iflytek.cloud.ui.control;

import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;

import com.iflytek.msc.MSC;

import android.content.Context;
import android.content.res.XmlResourceParser;
import android.graphics.drawable.Drawable;
import android.graphics.drawable.StateListDrawable;
import android.util.TypedValue;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

public class DrawableMgr {

	// private static final String TAG = "ResourceUtils";

	public static final String TAG_TITLE = "title";
	// recognizer
	public static final String TAG_CONTAINER = "container";
	public static final String TAG_CONTROL = "control";
	public static final String TAG_RECOGNIZE = "recognize";
	public static final String TAG_RECOGNIZE_BG = "voice_bg.9";
	
	//错误提示.
	public static final String TAG_ERROR	   = "error";
	public static final String TAG_ERROR_VIEW  = "errview";
	public static final String TAG_ERROR_TXT   = "errtxt";
	//识别提示语
	public static final String TAG_TIPS = "tips";

	public static final String TAG_LINK = "textlink";
	
	public static final String TAG_RECORDING_ANIMATION = "voice_";
	public static final String TAG_RECORDING = "recording";
	public static final String TAG_WAITING = "waiting";
	public static final String TAG_WARNING = "warning";
	
	public static final String TAG_INTERVAL_LINE = "<br>";
	
	//部分图片放在/iflytek文件夹下
	private static final String PATH_IFLYTEK = MSC.isIflyVersion() ? "iflytek/" : "cmcc/";
	
	private static final String PATH_ASSETS = "assets/" + PATH_IFLYTEK;
	
	private static final int  DENSITY = 240;
	
	private static HashMap<String,Drawable> sDraws     = new HashMap<String, Drawable>();
	private static HashMap<String,Drawable> sDrawLists = new HashMap<String, Drawable>();
	
	/**
	 * 错误文字颜色
	 * @return 错误提示，错误码颜色
	 */
	public static int[] getErrorTextColor() {
		return new int[]{0xffe7e7e7, 0xff686e72};
	}
	
	/**
	 * 错误文字字体大小
	 * @return 错误提示，错误码大小
	 */
	public static int[] getErrorTextSize() {
		return new int[]{20, 16};
	}
		
	private static InputStream getResourceStream(Context context,String path) throws IOException {
		InputStream stream = context.getAssets().open(path);
		return stream;
	}
	
	private static XmlResourceParser getResourceXml(Context context,String path) throws IOException {
		XmlResourceParser parser = context.getAssets().openXmlResourceParser(path);
		return parser;
	}

	public static View extractView(Context context, String name,
			ViewGroup root) throws Exception {
//		DebugLog.LogD("extractView = " + name);
		String filename = PATH_ASSETS + name + ".xml";
		XmlResourceParser parser = context.getAssets().openXmlResourceParser(filename);
		LayoutInflater inflater = (LayoutInflater) context
				.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
		return inflater.inflate(parser, root);
	}
	
	private static Drawable genStateListDrawable(Context context, String normal,
			String pressed, String focused, String disabled) throws Exception {
		StateListDrawable drawable = new StateListDrawable();
		if (null != disabled) {
			drawable.addState(new int[] { -android.R.attr.state_enabled },
					getDrawable(context, disabled));
		}
		if (null != pressed) {
			drawable.addState(new int[] { android.R.attr.state_pressed },
					getDrawable(context, pressed));
		}
		if (null != focused) {
			drawable.addState(new int[] { android.R.attr.state_focused },
					getDrawable(context, focused));
		}
		if (null != normal) {
			drawable.addState(new int[] {}, getDrawable(context, normal));
		}
		return drawable;
	}

	public static int SDK_CUPCAKE = 3;
	public static int SDK_DONUT = 4;
	public static int SDK_ECLAIR = 7;
	public static int SDK_FROYO = 8;
	
	public synchronized static Drawable getStateDrawList(Context context,
			String stateListTag, String[] stateList) throws Exception {
		String[] tempList = new String[4];
		Drawable draw = null;
		for(int i=0;i < stateList.length ;i++)
			tempList[i] = stateList[i];
	    if(draw == null){
	    	draw = genStateListDrawable(context, tempList[0], tempList[1], tempList[2], tempList[3]);
	    }
		return draw;
	}
	
	public synchronized static Drawable[] getDrawableList(Context context, String[] drawableList) throws Exception
	{
		 Drawable[] drawList = new Drawable[drawableList.length];
		 for(int i =0;i < drawableList.length;i++){
			 Drawable draw = sDrawLists.get(drawableList[i]);
			 if(draw == null ){
				 draw = getDrawable(context, drawableList[i]);
				 sDrawLists.put(drawableList[i], draw);
			 }
			 drawList[i] = draw;
		 }
	      return drawList;
	}
	public synchronized static Drawable getDrawable(Context context, String name) throws Exception
	{
		// DebugLog.Log("ResourceUtils", "getDrawable = " + name);
		Drawable draw = sDraws.get(name);
		if(draw == null)
		{
			draw = extractDrawable(context, name);
			sDraws.put(name, draw);
		}
		
		return draw;
	}
	
	public synchronized static Drawable getDrawableXml(Context context, String filename) throws Exception
	{
//		DebugLog.LogD("getDrawableXml = " + filename);
		return extractDrawableFromXml(context, filename);
	}
	
	private static Drawable extractDrawable(Context context, String name)
			throws Exception {
		// DebugLog.Log("ResourceUtils", "Extract Drawable = " + name);
		InputStream inputStream = getResourceStream(context, PATH_IFLYTEK + name + ".png");
		TypedValue value = new TypedValue();
		value.density = DENSITY;
		Drawable drawable = null;
		if (android.os.Build.VERSION.SDK_INT > SDK_CUPCAKE) {
			drawable = DrawableUtil.createFromResourceStream(
					context.getResources(), value, inputStream, name, null);
		} else {
			drawable = Drawable.createFromResourceStream(
					context.getResources(), value, inputStream, name);
		}
		if(inputStream != null)
			inputStream.close();
		return drawable;
	}

	private static Drawable extractDrawableFromXml(Context context,String fileName) 
	throws Exception {
		
		XmlResourceParser parser = getResourceXml(context,fileName);
		Drawable draw = Drawable.createFromXml(context.getResources(), parser);
		if(parser != null)
			parser.close();
		return draw;
	}
}
