package com.iflytek.cloud.ui.control;

import java.io.InputStream;

import android.content.res.Resources;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.BitmapFactory.Options;
import android.graphics.NinePatch;
import android.graphics.Rect;
import android.graphics.drawable.BitmapDrawable;
import android.graphics.drawable.Drawable;
import android.graphics.drawable.NinePatchDrawable;
import android.util.DisplayMetrics;
import android.util.TypedValue;

import com.iflytek.cloud.msc.util.log.DebugLog;

public class DrawableUtil {

	// private static final String TAG = "DrawableUtils";
	@SuppressWarnings("unused")
	private static int avoidValue = 0;

	// avoid 1.5 system error,can't delete
	public void avoidSystemError(int value) {
		DebugLog.LogD("avoidSystemError");
		avoidValue = value;
	}
	/**
	 * Create a drawable from an inputstream, using the given resources and
	 * value to determine density information.
	 */
	public static Drawable createFromResourceStream(Resources res,
			TypedValue value, InputStream is, String srcName,
			BitmapFactory.Options opts) {

		if (is == null) {
			return null;
		}
		/*
		 * ugh. The decodeStream contract is that we have already allocated the
		 * pad rect, but if the bitmap does not had a ninepatch chunk, then the
		 * pad will be ignored. If we could change this to lazily alloc/assign
		 * the rect, we could avoid the GC churn of making new Rects only to
		 * drop them on the floor.
		 */
		Rect pad = new Rect();

		// Special stuff for compatibility mode: if the target density is not
		// the same as the display density, but the resource -is- the same as
		// the display density, then don't scale it down to the target density.
		// This allows us to load the system's density-correct resources into
		// an application in compatibility mode, without scaling those down
		// to the compatibility density only to have them scaled back up when
		// drawn to the screen.
		if (opts == null)
			opts = new BitmapFactory.Options();
		// opts.inScreenDensity = 240;
		Bitmap bm = decodeResourceStream(res, value, is, pad, opts);
		if (bm != null) {
			byte[] np = bm.getNinePatchChunk();
			if (np == null || !NinePatch.isNinePatchChunk(np)) {
				np = null;
				pad = null;
			}
			Drawable drw = drawableFromBitmap(res, bm, np, pad, srcName);
			return drw;
		}
		return null;
	}

	/**
	 * Decode a new Bitmap from an InputStream. This InputStream was obtained
	 * from resources, which we pass to be able to scale the bitmap accordingly.
	 */
	public static Bitmap decodeResourceStream(Resources res, TypedValue value,
			InputStream is, Rect pad, Options opts) {

		if (opts == null) {
			opts = new Options();
		}

		if (opts.inDensity == 0 && value != null) {
			final int density = value.density;
			if (density == TypedValue.DENSITY_DEFAULT) {
				opts.inDensity = DisplayMetrics.DENSITY_DEFAULT;
			} else if (density != TypedValue.DENSITY_NONE) {
				opts.inDensity = density;
			}
		}

		if (opts.inTargetDensity == 0 && res != null) {
			opts.inTargetDensity = res.getDisplayMetrics().densityDpi;
		}

		return BitmapFactory.decodeStream(is, pad, opts);
	}

	private static Drawable drawableFromBitmap(Resources res, Bitmap bm,
			byte[] np, Rect pad, String srcName) {

		if (np != null) {
			return new NinePatchDrawable(res, bm, np, pad, srcName);
		}

		return new BitmapDrawable(res, bm);
	}
}
