
package com.iflytek.cloud.util.contacts;

import android.content.Context;
import android.telephony.PhoneNumberUtils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import com.iflytek.cloud.util.contacts.entities.ContactItem;
import com.iflytek.cloud.util.contacts.entities.ContactSet;
import com.iflytek.cloud.util.contacts.interfaces.ContactAccessor;


/**
 * 联系人辅助类
 * @author weiwei
 *
 */

public class ContactHelper {
	
	private static final String UNKNOWN_CONTACT = "未知联系人";
    
    private ContactAccessor mAccessor;
    
    protected Context mContext;
    
    private String[] mContactNames = null;
    
    private String[] mContactInfos = null; 
    

/**
     * <ID+P/S,Name> 保存联系人名字对应的ID;一个名字可能多个ID;
     */

    private HashMap<String, String> mContactNameHashMap = new HashMap<String, String>();
    

/**
     * <ID+S,Address> 保存SIM卡号码与对应的ID;通过ID可以从 mContactNameHashMap 取出Name,ID通过后缀Phone/SIM区分位置;
     * Cache只保存基本信息,如果需要完成信息,需要使用ID从相应的表中查询; 
     */

    private HashMap<String, String> mSimContactAddress = new HashMap<String, String>();
    

/**
     * SIM卡联系人缓存
     */
    private List<ContactItem> mSimContactCache = new ArrayList<ContactItem>();
    

/**
     * 含有特殊字符‘\0'的map表，将特殊字符过滤为空格 
     */

    private HashMap<String, String> mSpecCharHashMap = new HashMap<String, String>();
    
    public ContactHelper(Context context, ContactAccessor contactAccessor) {
        mAccessor = contactAccessor;
        mContext = context;
    }
    
    public ContactHelper(Context context) {
    	mContext = context;
    }
    

/**
     * 通过号码从Cache中查询ID
     * 
     * @param number
     * @return
     */

    private String getContactIdByNumber(String number) {
        String ret_id = null;
        String tmp_id = null;
        if (null == number) {
            return null;
        }

        Set<String> key_numbers = mSimContactAddress.keySet();
        // 全匹配
        for (String tmp_id_flag : key_numbers) {
            if (number.equals(mSimContactAddress.get(tmp_id_flag))) {
                if (tmp_id_flag.endsWith("p")) {
                    ret_id = tmp_id_flag;
                    break;
                } else {
                    tmp_id = tmp_id_flag;
                }   
            }
        }
        if (null != tmp_id && null == ret_id) {
            return tmp_id;
        }
        // 查不到,部分匹配
        if (null == ret_id && null == tmp_id) {
            for (String tmp_id_flag : key_numbers) {
                if (PhoneNumberUtils.compare(mSimContactAddress.get(tmp_id_flag), number)) {
                    if (tmp_id_flag.endsWith("p")) {
                        ret_id = tmp_id_flag;
                        break;
                    } else {
                        tmp_id = tmp_id_flag;
                    }   
                }
            } 
            if (null != tmp_id && null == ret_id) {
                return tmp_id;
            }
        }

        return ret_id;
    }
    
    public String[] getContactNameCache() {
        return mContactNames;
    }
    

/**
     * 获取所有联系人名称
     * @return String[]
     */

    public String[] getAllContactNames() {
        clearAllCache();
        List<String> nameList = new ArrayList<String>();

        HashMap<String, String> nameHashMap = mAccessor.queryContactNames();
        List<ContactItem> simItemList = mAccessor.querySIMContactInfo();
        
        Iterator<String> iterator = nameHashMap.keySet().iterator();
        while (iterator.hasNext()) {
            String id = iterator.next();
            String name = nameHashMap.get(id);
            mContactNameHashMap.put(id + "p", name);
            nameList.add(name);
            // 为了解决java字符串中存在的\0在C代码中会截断的问题，作特殊字符的缓存映射表
            if (name.contains("\0")) {
                String tmpName = name.replace("\0", " ");
                mSpecCharHashMap.put(tmpName, name);
            }
        }
        for (ContactItem item : simItemList) {
            String id = item.getContactId();
            String number = item.getPhoneNumber();
            String name = item.getContactName();
            mSimContactAddress.put(id + "s", number);
            mContactNameHashMap.put(id + "s", name);
            nameList.add(name);
             // 为了解决java字符串中存在的\0在C代码中会截断的问题，作特殊字符的缓存映射表 
            if (name.contains("\0")) {
                String tmpName = name.replace("\0", " ");
                mSpecCharHashMap.put(tmpName, name);
            }
            mSimContactCache.add(item);
        }

        // 去除重复的联系人名称
        Set<String> tmpSet = new HashSet<String>(nameList);
        mContactNames = tmpSet.toArray(new String[tmpSet.size()]);

        return mContactNames;
    }
    

/**
     * 获取所有联系人信息，现版本包括名称，号码
     * @return String[]
     */

    public String[] getAllContactInfo() {
        clearAllCache();
        List<String> infoList = new ArrayList<String>();

        HashMap<String, String> infoHashMap = mAccessor.queryContactInfo();
        List<ContactItem> simItemList = mAccessor.querySIMContactInfo();
        
        Iterator<String> iterator = infoHashMap.keySet().iterator();
        while (iterator.hasNext()) {
            String id = iterator.next();
            String info = infoHashMap.get(id);
            infoList.add(info);
        }
        for (ContactItem item : simItemList) {
            String number = item.getPhoneNumber();
	        String name = item.getContactName();
	        //暂时不做过滤操作，如需过滤可将key设置为name
//	        if(!infoHashMap.containsKey(name))
//	        {
	        String sim_info = name + "#" + number;
	        infoList.add(sim_info);
//            }
        }

        // 去除重复的联系人名称
        Set<String> tmpSet = new HashSet<String>(infoList);
        mContactInfos = tmpSet.toArray(new String[tmpSet.size()]);

        return mContactInfos;
    }
    

/**
     * 根据号码获取联系人名称
     * @param number
     * @return
     */

    public String getContactNameByNumber(String number) {
        String nameStr = null;
        if (null != number) {
            number = PhoneNumberUtil.filterHeadNumber(number);
            nameStr = mAccessor.queryContactNameByNumber(number);
            if (null == nameStr) {
                if (mSimContactAddress.size() > 0) {
                    String id = getContactIdByNumber(number);
                    if (null != id) {
                        nameStr = mContactNameHashMap.get(id);
                    }
                }
            }
        }

        return nameStr;
    }
    

/**
     * 根据名称查询sim卡中联系人信息
     * @param name
     * @return ContactSet
     */

    public ContactSet querySimContactSetByName(String name) {
        ContactSet set = null;
        ArrayList<String> numberList = new ArrayList<String>();
        ArrayList<String> tmpList = new ArrayList<String>();
        HashMap<String, String> numberTypeHashMap = new HashMap<String, String>();
        
        if (null != name) {
            for (ContactItem item : mSimContactCache) {
                if (item.getContactName().equals(name)) {
                    tmpList.add(item.getPhoneNumber());
                    numberTypeHashMap.put(item.getPhoneNumber(), item.getNumberType());
                }
            }
            if (tmpList.size() > 0) {
                set = new ContactSet(name, null);
                numberList = new ArrayList<String>(new HashSet<String>(tmpList));
                set.setNumbers(numberList);
                set.setNumberTypeHashMap(numberTypeHashMap);
            }
        }
        return set;
    }
    
    

/**
     * 根据姓名查询ContactSet集合
     * @param nameList
     * @param fuzzyFlag 是否需要模糊匹配姓名
     * @return ArrayList<ContactSet>
     */

    public ArrayList<ContactSet> queryContactSetByNames(boolean fuzzyFlag, List<String> nameList) {
        ArrayList<ContactSet> queryContactSets = new ArrayList<ContactSet>();

        if (null != nameList && nameList.size() > 0) {
            for (String name : nameList) {
                ContactSet set = null; 
                ArrayList<ContactSet> tmpContactSets = new ArrayList<ContactSet>();
                ArrayList<String> numbers = new ArrayList<String>();
                if (mSpecCharHashMap.containsKey(name)) {
                    name = mSpecCharHashMap.get(name);
                }
                
                if (fuzzyFlag) {
                    tmpContactSets = mAccessor.fuzzyQueryContactSets(null, name);
                } else {
                    set = mAccessor.queryContactSet(null, name);
                }
                
                if (null == set && 0 == tmpContactSets.size()) {
                    // 如果没有联系人，从sim卡中读取
                    set = querySimContactSetByName(name);
                    if (null == set) {
                        // 未知联系人
                        if (null != name && PhoneNumberUtil.isNumeric(PhoneNumberUtil.filterHeadNumber(name))) {
                            set = new ContactSet(UNKNOWN_CONTACT, null);
                            numbers.add(name);
                            set.setNumbers(numbers);
                        }
                    }
                    if (null != set) {
                        queryContactSets.add(set);
                    }
                } else if (tmpContactSets.size() > 0) {
                    queryContactSets.addAll(tmpContactSets);
                } else {
                    queryContactSets.add(set);
                }

            }
        }
        return queryContactSets;
    }
    
    

/**
     * 如果识别结果是号码，查询联系人信息
     * @param numberList
     * @return 返回ContactSet列表
     */

    public List<ContactSet> getContactSetsByNumber(List<String> numberList) {
        List<ContactSet> contactSetList = new ArrayList<ContactSet>();
        if (null != numberList && numberList.size() > 0) {
            contactSetList = new ArrayList<ContactSet>();
            for (String number : numberList) {
                ContactSet set = null;
                String name = ContactManagerImpl.getManager().queryContactNameByNumber(number);
                if (null != name) {
                    set = new ContactSet(name, null);
                } else {
                    set = new ContactSet(UNKNOWN_CONTACT, null);
                }
                ArrayList<String> tmpNumberList = new ArrayList<String>();
                tmpNumberList.add(number);
                set.setNumbers(tmpNumberList);
                contactSetList.add(set);
            }
        }
        return contactSetList;
    }
    

/**
     * 检查联系人查询是否有结果，true表示有结果，false表示没有结果
     * @param contactSets
     * @return
     */

    public boolean checkContactResult(List<ContactSet> contactSets) {
    	if (null == contactSets || contactSets.size() == 0) {
    		return false;
    	}
    	return true;
    }
    

/**
     * 过滤没有号码的联系人
     * @param list
     * @return
     */

    public ArrayList<ContactSet> filterNoNumberContactSets(ArrayList<ContactSet> list) {
        ArrayList<ContactSet> tmpArrayList = new ArrayList<ContactSet>();
        if (null != list && list.size() > 0) {
            for (ContactSet set : list) {
                if (null == set.getNumbers() || 0 == set.getNumbers().size()) {
                    continue;
                } else {
                    tmpArrayList.add(set);
                }
            }
        }
        return tmpArrayList;
    }
    
    
    
    public boolean hasPhoneNumber(String name) {
        if (null == name) {
            return false;
        }
        
        boolean hasNumberFlag = false;
        if (mAccessor.hasContactNumber(name)) {
            hasNumberFlag = true;
        } else {
            if (mSimContactCache.size() > 0) {
                for (ContactItem item : mSimContactCache) {
                    if (name.equals(item.getContactName()) && null != item.getPhoneNumber()) {
                        hasNumberFlag = true;
                        break;
                    }
                }
            }
        }
        return hasNumberFlag;
    }
    

/**
     * 清除联系人相关缓存
     */

    private void clearAllCache() {
        if (mSimContactAddress.size() > 0) {
            mSimContactAddress = null;
            mSimContactAddress =  new HashMap<String, String>();
        }
        
        if (mSpecCharHashMap.size() > 0) {
            mSpecCharHashMap = null;
            mSpecCharHashMap = new HashMap<String, String>();
        }
        
        if (mContactNameHashMap.size() > 0) {
            mContactNameHashMap = null;
            mContactNameHashMap = new HashMap<String, String>();
        }
        
        if (null != mContactNames && mContactNames.length > 0) {
            mContactNames = null;
        }
    }
    

/**
     * 根据联系人姓名查询头像信息
     * @param name
     * @return
     */

	public byte[] queryContactPhotoData(String name) {
		if (null == name) {
			return null;
		}
		return mAccessor.queryContactPhotoDataByName(name);
	}
	

/**
	 * 根据号码和姓名直接往数据库插入联系人
	 * @param name
	 * @param number
	 */

	public void insertContact(String name, String number) {
		mAccessor.insertNewContact(name, number);
	}
	
}

